@@ -42,7 +42,7 @@ file_data_new (const char       *path,
 	fd = g_new0 (FileData, 1);
 
 	fd->ref = 1;
-	fd->path = g_strdup (path);
+	fd->path = get_uri_from_path (path);
 	fd->name = file_name_from_path (fd->path);
 	fd->utf8_name = g_filename_to_utf8 (fd->name, -1, 0, 0, 0);
 	if (info != NULL) {
@@ -117,7 +117,7 @@ file_data_update_comment (FileData *fd)
 	if (fd->comment != NULL)
 		g_free (fd->comment);
 
-	data = comments_load_comment (fd->path);
+	data = comments_load_comment (fd->path, FALSE);
 
 	if (data == NULL) {
 		fd->comment = g_strdup ("");
@@ -156,6 +156,24 @@ file_data_unref (FileData *fd)
 }
 
 
+GList*
+file_data_list_dup (GList *list)
+{
+	GList *new_list = NULL, *scan;
+
+	if (list == NULL)
+		return NULL;
+
+	for (scan = list; scan; scan = scan->next) {
+		FileData *data = scan->data;
+		file_data_ref (data);
+		new_list = g_list_prepend (new_list, data);
+	}
+
+	return g_list_reverse (new_list);
+}
+
+
 void
 file_data_list_free (GList *list)
 {
