@@ -55,7 +55,7 @@
 
 #define MAX_NAME_LEN 1024
 #define DEF_CONFIRM_DEL TRUE
-
+#define CATALOG_PERMISSIONS 0600
 
 void
 gth_browser_activate_action_file_new_window (GtkAction  *action,
@@ -68,7 +68,7 @@ gth_browser_activate_action_file_new_window (GtkAction  *action,
 	switch (gth_browser_get_sidebar_content (browser)) {
 	case GTH_SIDEBAR_DIR_LIST:
 		path = gth_browser_get_current_directory (browser);
-		uri = g_strconcat ("file://", path, NULL);
+		uri = get_uri_from_path (path);
 		break;
 
 	case GTH_SIDEBAR_CATALOG_LIST:
@@ -219,7 +219,7 @@ rename_file (GtkWindow   *window,
 
 	all_windows_remove_monitor ();
 
-	if (strcmp (old_path, new_path) == 0) {
+	if (same_uri (old_path, new_path)) {
 		char *utf8_path;
 
 		utf8_path = g_filename_display_name (old_path);
@@ -562,7 +562,7 @@ gth_browser_activate_action_edit_catalog_view_new_window (GtkAction  *action,
 	path = catalog_list_get_selected_path (gth_browser_get_catalog_list (browser));
 	if (path == NULL) 
 		return;
-	uri = g_strconcat ("catalog://", path, NULL);
+	uri = g_strconcat ("catalog://", remove_scheme_from_uri (path), NULL);
 
 	gtk_widget_show (gth_browser_new (uri));
 
@@ -641,7 +641,7 @@ catalog_rename (GthBrowser *browser,
 				       _("The name \"%s\" is already used. " "Please use a different name."), utf8_name);
 		g_free (utf8_name);
 
-	} else if (! rename (catalog_path, new_catalog_path)) {
+	} else if (gnome_vfs_move (catalog_path, new_catalog_path, TRUE) == GNOME_VFS_OK) {
 		all_windows_notify_catalog_rename (catalog_path, 
 						   new_catalog_path);
 	} else {
@@ -853,7 +853,7 @@ gth_browser_activate_action_edit_current_catalog_new (GtkAction  *action,
 				       _("The name \"%s\" is already used. " "Please use a different name."), utf8_name);
 		g_free (utf8_name);
 
-	} else if (gnome_vfs_create (&vfs_handle, new_catalog_path, GNOME_VFS_OPEN_WRITE, FALSE, 0660) == GNOME_VFS_OK) {
+	} else if (gnome_vfs_create (&vfs_handle, new_catalog_path, GNOME_VFS_OPEN_WRITE, FALSE, CATALOG_PERMISSIONS) == GNOME_VFS_OK) {
 		gnome_vfs_close (vfs_handle);
 		all_windows_notify_catalog_new (new_catalog_path);
 
@@ -933,8 +933,7 @@ create_new_folder_or_library (GthBrowser *browser,
 				       _("The name \"%s\" is already used. " "Please use a different name."), utf8_name);
 		g_free (utf8_name);
 
-	} else if (mkdir (new_path, 0755) == 0) { 
-	} else {
+	} else if (! dir_make (new_path, 0755)) { 
 		char *utf8_path;
 
 		utf8_path = g_filename_display_name (new_path);
@@ -1045,8 +1044,8 @@ gth_browser_activate_action_edit_dir_view_new_window (GtkAction  *action,
 	path = dir_list_get_selected_path (gth_browser_get_dir_list (browser));
 	if (path == NULL) 
 		return;
-	uri = g_strconcat ("file://", path, NULL);
 
+	uri = get_uri_from_path (path);
 	gtk_widget_show (gth_browser_new (uri));
 
 	g_free (uri);
@@ -1064,7 +1063,7 @@ show_folder (GtkWindow  *window,
 	if (path == NULL)
 		return;
 	
-	uri = g_strconcat ("file://", path, NULL);
+	uri = get_uri_from_path (path);
 	result = gnome_vfs_url_show (uri);
 	if (result != GNOME_VFS_OK)
 		_gtk_error_dialog_run (window, gnome_vfs_result_to_string (result));
@@ -1138,7 +1137,7 @@ folder_rename (GtkWindow  *window,
 
 	all_windows_remove_monitor ();
 	
-	if (strcmp (old_path, new_path) == 0) {
+	if (same_uri (old_path, new_path)) {
 		char *utf8_path;
 		
 		utf8_path = g_filename_display_name (old_path);
@@ -1162,13 +1161,13 @@ folder_rename (GtkWindow  *window,
 
 		old_folder_comment = comments_get_comment_filename (old_path, TRUE, TRUE);
 
-		if (rename (old_path, new_path) == 0) { 
+		if (gnome_vfs_move (old_path, new_path, TRUE) == GNOME_VFS_OK) { 
 			char *new_folder_comment;
 
 			/* Comment cache. */
 
 			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
-			rename (old_folder_comment, new_folder_comment);
+			gnome_vfs_move (old_folder_comment, new_folder_comment, TRUE);
 			g_free (new_folder_comment);
 			
 			all_windows_notify_directory_rename (old_path, new_path);
@@ -1407,7 +1406,7 @@ folder_copy__response_cb (GObject *object,
 
 	message = move ? _("Could not move the folder \"%s\": %s") : _("Could not copy the folder \"%s\": %s");
 
-	if (strcmp (old_path, new_path) == 0) {
+	if (same_uri (old_path, new_path)) {
 		char *utf8_path;
 		
 		utf8_path = g_filename_display_name (old_path);
@@ -1457,14 +1456,14 @@ folder_copy__response_cb (GObject *object,
 			
 		old_folder_comment= comments_get_comment_filename (old_path, TRUE, TRUE);
 
-		if (rename (old_path, new_path) == 0) { 
+		if (gnome_vfs_move (old_path, new_path, TRUE) == GNOME_VFS_OK) { 
 			char *new_folder_comment;
 
 			/* moving folders on the same file system can be 
 			 * implemeted with rename, which is faster. */
 
 			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
-			rename (old_folder_comment, new_folder_comment);
+			gnome_vfs_move (old_folder_comment, new_folder_comment, TRUE);
 			g_free (new_folder_comment);
 
 			all_windows_notify_directory_rename (old_path, new_path);
@@ -1818,16 +1817,48 @@ gth_browser_activate_action_go_delete_history (GtkAction  *action,
 }
 
 
+static void
+open_location_response_cb (GtkDialog  *file_sel,
+			   int         button_number,
+			   gpointer    data)
+{
+	GthBrowser *browser = data;
+	const char *folder;
+
+	if (button_number != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (file_sel));
+		return;
+	}
+
+	folder = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (file_sel));
+	if (folder != NULL)
+		gth_browser_go_to_directory (browser, folder);
+	gtk_widget_destroy (GTK_WIDGET (file_sel));
+}
+
+
 void
 gth_browser_activate_action_go_location (GtkAction  *action,
 					 GthBrowser *browser)
 {
-	/* FIXME
-	GThumbWindow *window = data;
-	if (! window->sidebar_visible)
-		window_show_sidebar (window);
-	gtk_widget_grab_focus (window->location_entry);
-	*/
+	GtkWidget *chooser;
+
+	chooser = gtk_file_chooser_dialog_new (_("Open Location"),
+					       GTK_WINDOW (browser),
+					       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					       NULL);
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (chooser), gth_browser_get_current_directory (browser));
+
+	g_signal_connect (G_OBJECT (GTK_DIALOG (chooser)),
+			  "response", 
+			  G_CALLBACK (open_location_response_cb), 
+			  browser);
+
+	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	gtk_widget_show (chooser);
 }
 
 
@@ -1850,9 +1881,8 @@ gth_browser_activate_action_bookmarks_add (GtkAction  *action,
 			prefix = g_strdup (CATALOG_PREFIX);
 		catalog_path = catalog_list_get_path_from_iter (catalog_list, &iter);
 		
-		path = g_strconcat (prefix,
-				    catalog_path,
-				    NULL);
+		path = g_strconcat (prefix, remove_scheme_from_uri (catalog_path), NULL);
+
 		g_free (catalog_path);
 		g_free (prefix);
 
@@ -1861,9 +1891,7 @@ gth_browser_activate_action_bookmarks_add (GtkAction  *action,
 
 		if (dir_list->path == NULL)
 			return;
-		path = g_strconcat (FILE_PREFIX,
-				    dir_list->path,
-				    NULL);
+		path = get_uri_from_path (dir_list->path);
 	}
 	
 	if (path == NULL) 
