@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,147 +27,107 @@
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libbonobo.h>
 
-#include "actions.h"
-#include "main.h"
-#include "auto-completion.h"
 #include "catalog.h"
-#include "dir-list.h"
-#include "file-utils.h"
-#include "fullscreen.h"
-#include "gconf-utils.h"
-#include "gthumb-init.h"
-#include "gthumb-window.h"
-#include "image-viewer.h"
-#include "gth-image-list.h"
-#include "preferences.h"
-#include "icons/pixbufs.h"
-#include "typedefs.h"
 #include "comments.h"
+#include "dir-list.h"
+#include "main.h"
+#include "file-utils.h"
+#include "gconf-utils.h"
+#include "gtk-utils.h"
+#include "gth-application.h"
+#include "gth-browser.h"
+#include "gth-browser-actions-callbacks.h"
+#include "gth-image-list.h"
+#include "gth-viewer.h"
+#include "gthumb-init.h"
+#include "gthumb-stock.h"
+#include "image-viewer.h"
 #include "pixbuf-utils.h"
+#include "preferences.h"
+#include "typedefs.h"
 
+#include "icons/pixbufs.h"
 
-#define ICON_NAME_DIRECTORY "gnome-fs-directory"
+typedef enum {
+	ICON_NAME_DIRECTORY,
+	ICON_NAME_HARDDISK,
+	ICON_NAME_HOME,
+	ICON_NAME_DESKTOP,
+	ICON_NAMES
+} IconName;
 
+static char*icon_mime_name[ICON_NAMES] = { "gnome-fs-directory",
+					   "gnome-dev-harddisk",
+					   "gnome-fs-home",
+					   "gnome-fs-desktop" };
 
-GList          *window_list = NULL;
-FullScreen     *fullscreen;
-char          **file_urls, **dir_urls;
-int             n_file_urls, n_dir_urls;
-int             StartInFullscreen = FALSE;
-int             StartSlideshow = FALSE;
-int             ViewFirstImage = FALSE;
-int             HideSidebar = FALSE;
-gboolean        ExitAll = FALSE;
-char           *ImageToDisplay = NULL;
-gboolean        FirstStart = TRUE;
-gboolean        ImportPhotos = FALSE;
+GthMonitor  *monitor = NULL;
+char       **file_urls, **dir_urls;
+int          n_file_urls, n_dir_urls;
+int          StartInFullscreen = FALSE;
+int          StartSlideshow = FALSE;
+int          ViewFirstImage = FALSE;
+int          HideSidebar = FALSE;
+gboolean     ExitAll = FALSE;
+char        *ImageToDisplay = NULL;
+gboolean     FirstStart = TRUE;
+gboolean     ImportPhotos = FALSE;
 
 static gboolean        view_comline_catalog = FALSE;
 static gboolean        view_single_image = FALSE;
-static GdkPixbuf      *folder_pixbuf = NULL;
-static GThumbWindow   *first_window = NULL;
+static GdkPixbuf      *icon_pixbuf[ICON_NAMES] = { 0 };
+static GtkWidget      *first_window = NULL;
 static GnomeIconTheme *icon_theme = NULL;
-
-
-static void     prepare_app         ();
-static void     initialize_data     (poptContext pctx);
-static void     release_data        ();
-
-static void     init_session        (const char *argv0);
-static gboolean session_is_restored (void);
-static gboolean load_session        (void);
-
+static BonoboObject   *gth_application = NULL;
 
 struct poptOption options[] = {
 	{ "fullscreen", '\0', POPT_ARG_NONE, &StartInFullscreen, 0,
 	  N_("Start in fullscreen mode"),
 	  0 },
-
 	{ "slideshow", '\0', POPT_ARG_NONE, &StartSlideshow, 0,
 	  N_("Automatically start a slideshow"),
 	  0 },
-
 	{ "import-photos", '\0', POPT_ARG_NONE, &ImportPhotos, 0,
 	  N_("Automatically import digital camera photos"),
 	  0 },
-
 	{ NULL, '\0', 0, NULL, 0 }
 };
 
 
-/* -- Main -- */
-
-
-static gboolean 
-check_whether_to_set_fullscreen (gpointer data) 
+static void
+init_icon_pixbufs (void)
 {
-	if ((first_window != NULL) && StartInFullscreen) 
-		fullscreen_start (fullscreen, first_window);
-	StartInFullscreen = FALSE;
-	return FALSE;
-}
-
-
-static gboolean 
-check_whether_to_import_photos (gpointer data) 
-{
-	if ((first_window != NULL) && ImportPhotos) 
-		activate_action_file_camera_import (NULL, first_window);
-	return FALSE;
-}
-
-
-int 
-main (int argc, char *argv[])
-{
-	GnomeProgram *program;
-	GValue        value = { 0 };
-	poptContext   pctx;
-
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gthumb", VERSION,
-				      LIBGNOMEUI_MODULE, 
-				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
-				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
-				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
-				      GNOME_PARAM_APP_DATADIR, GTHUMB_DATADIR,
-				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
-				      NULL);
-
-	if (! g_thread_supported ()) {
-		g_thread_init (NULL);
-		gdk_threads_init ();
-	}
-
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-	pctx = g_value_get_pointer (&value);
-	glade_gnome_init ();
-	gthumb_init ();
-	initialize_data (pctx);
-	poptFreeContext (pctx);
-	prepare_app ();
-
-	g_idle_add (check_whether_to_set_fullscreen, NULL);
-	g_idle_add (check_whether_to_import_photos, NULL);
-
-	gtk_main ();
-
-	release_data ();
-
-	return 0;
+	int i;
+	for (i = 0; i < ICON_NAMES; i++) 
+		icon_pixbuf[i] = NULL;
 }
 
 
 static void
-create_default_categories_if_needed ()
+free_icon_pixbufs (void)
+{
+	int i;
+	for (i = 0; i < ICON_NAMES; i++)
+		if (icon_pixbuf[i] != NULL) {
+			g_object_unref (icon_pixbuf[i]);
+			icon_pixbuf[i] = NULL;
+		}
+}
+
+
+static void
+theme_changed_cb (GnomeIconTheme *theme, 
+		  gpointer        data)
+{
+	free_icon_pixbufs ();
+	all_windows_notify_update_icon_theme ();
+}
+
+
+static void
+create_default_categories_if_needed (void)
 {
 	Bookmarks *categories;
 	char      *default_categories[] = { N_("Holidays"), 
@@ -204,18 +164,6 @@ create_default_categories_if_needed ()
 }
 
 
-static void
-theme_changed_cb (GnomeIconTheme *theme, 
-		  gpointer        data)
-{
-	if (folder_pixbuf != NULL) {
-		g_object_unref (folder_pixbuf);
-		folder_pixbuf = NULL;
-	}
-	all_windows_notify_update_icon_theme ();
-}
-
-
 static void 
 convert_old_comment (char     *real_file, 
 		     char     *rc_file, 
@@ -235,9 +183,6 @@ convert_old_comment (char     *real_file,
 }
 
 
-
-
-
 static void
 convert_to_new_comment_system (void)
 {
@@ -255,642 +200,69 @@ convert_to_new_comment_system (void)
 }
 
 
-/* Initialize application data. */
-static void 
-initialize_data (poptContext pctx)
-{
-	const char **argv;
-	int          argc;
-	char        *current_dir;
-	char        *path;
-	char        *pixmap_file;
-	int          i;
-
-	convert_to_new_comment_system (); /* FIXME */
-	create_default_categories_if_needed ();
-
-	eel_gconf_monitor_add ("/apps/gthumb");
-
-	fullscreen = fullscreen_new ();
-
-	pixmap_file = PIXMAPSDIR "gthumb.png";
-	if (g_file_test (pixmap_file, G_FILE_TEST_EXISTS))
-		gnome_window_icon_set_default_from_file (pixmap_file);
-
-	icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
-	g_signal_connect (icon_theme,
-			  "changed",
-			  G_CALLBACK (theme_changed_cb),
-			  NULL);
-
-	init_session ("gthumb");
-	if (session_is_restored ()) 
-		return;
-
-	/* Parse command line arguments. */
-
-	argv = poptGetArgs (pctx);
-
-	if (argv == NULL)
-		return;
-
-	for (argc = 0; argv[argc] != NULL; argc++) 
-		;
-
-	if (argc == 0)
-		return;
-
-	file_urls = g_new0 (char*, argc);
-	dir_urls = g_new0 (char*, argc);
-
-	n_file_urls = 0;
-	n_dir_urls = 0;
-
-	current_dir = g_get_current_dir ();
-	for (i = 0; i < argc; i++) {
-		char     *tmp1, *tmp2;
-		gboolean  is_dir;
-
-		if (! g_path_is_absolute (argv[i]))
-			tmp1 = g_strconcat (current_dir, "/", argv[i], NULL);
-		else
-			tmp1 = g_strdup (argv[i]);
-
-		tmp2= remove_special_dirs_from_path (tmp1);
-
-		if (! g_path_is_absolute (tmp2))
-			path = g_strconcat (current_dir, "/", tmp2, NULL);
-		else {
-			path = tmp2;
-			tmp2 = NULL;
-		}
-		
-		g_free (tmp1);
-		g_free (tmp2);
-
-		if (path_is_dir (path))
-			is_dir = TRUE;
-		else if (path_is_file (path))
-			is_dir = FALSE;
-		else {
-			g_free (path);
-			continue;
-		}
-
-		if (is_dir) {
-			dir_urls[n_dir_urls++] = g_strconcat ("file://",
-							      path,
-							      NULL);
-			g_free (path);
-		} else
-			file_urls[n_file_urls++] = path;
-	}
-
-	if ((n_file_urls == 1) && (file_urls[0] != NULL)) {
-		view_single_image = TRUE;
-
-	} else if (n_file_urls > 1) {
-		/* Create a catalog with the command line list. */
-		Catalog *catalog;
-		char    *catalog_path;
-		char    *catalog_name, *catalog_name_utf8;
-
-		catalog = catalog_new ();
-
-		catalog_name_utf8 = g_strconcat (_("Command Line"),
-						 CATALOG_EXT,
-						 NULL);
-		catalog_name = g_filename_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
-		catalog_path = get_catalog_full_path (catalog_name);
-		g_free (catalog_name);
-		g_free (catalog_name_utf8);
-
-		catalog_set_path (catalog, catalog_path);
-		g_free (catalog_path);
-
-		for (i = 0; i < n_file_urls; i++) 
-			catalog_add_item (catalog, file_urls[i]);
-
-		catalog_write_to_disk (catalog, NULL);
-		catalog_free (catalog);
-
-		view_comline_catalog = TRUE;
-	}
-
-	g_free (current_dir);
-}
-
-
-/* Free application data. */
-static void 
-release_data ()
-{
-	if (folder_pixbuf != NULL)
-		g_object_unref (folder_pixbuf);
-
-	g_object_unref (icon_theme);
-
-	fullscreen_close (fullscreen);
-	preferences_release ();
-	eel_global_client_free ();
-	auto_compl_reset ();
-}
-
-
-/* Create the windows. */
-static void 
-prepare_app ()
-{
-	GThumbWindow *current_window = NULL;
-	int           i;
-	
-	if (session_is_restored ()) {
-		load_session ();
-		return;
-	}
-
-	if (! view_comline_catalog 
-	    && (n_dir_urls == 0) 
-	    && (n_file_urls == 0)) {
-		if (ImportPhotos)
-			preferences_set_startup_location (NULL);
-		current_window = window_new ();
-		if (!ImportPhotos)
-			gtk_widget_show (current_window->app);
-		if (first_window == NULL)
-			first_window = current_window;
-	}
-
-	if (view_single_image) {
-		char *image_folder;
-
-		ImageToDisplay = g_strdup (file_urls[0]);
-		HideSidebar = TRUE;
-
-		image_folder = remove_level_from_path (ImageToDisplay);
-		preferences_set_startup_location (image_folder);
-		g_free (image_folder);
-
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
-
-		if (first_window == NULL)
-			first_window = current_window;
-
-	} else if (view_comline_catalog) {
-		char *catalog_uri;
-		char *catalog_path;
-		char *catalog_name;
-
-		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
-		catalog_path = get_catalog_full_path (catalog_name);
-		g_free (catalog_name);
-
-		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
-		g_free (catalog_path);
-
-		preferences_set_startup_location (catalog_uri);
-		g_free (catalog_uri);
-
-		ViewFirstImage = TRUE;
-		HideSidebar = TRUE;
-
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
-
-		if (first_window == NULL)
-			first_window = current_window;
-	}
-
-	for (i = 0; i < n_dir_urls; i++) {
-		/* Go to the specified directory. */
-		preferences_set_startup_location (dir_urls[i]);
-
-		current_window = window_new ();
-		gtk_widget_show (current_window->app);
-
-		if (first_window == NULL)
-			first_window = current_window;
-	}
-
-	/* Free urls. */
-
-	if (n_file_urls > 0) {
-		for (i = 0; i < n_file_urls; i++)
-			g_free (file_urls[i]);
-		g_free (file_urls);
-	}
-
-	if (n_dir_urls > 0) {
-		for (i = 0; i < n_dir_urls; i++)
-			g_free (dir_urls[i]);
-		g_free (dir_urls);
-	}
-}
-
-
-void 
-all_windows_update_file_list ()
-{
-	g_list_foreach (window_list, (GFunc) window_update_file_list, NULL);
-}
-
-
-void 
-all_windows_update_catalog_list ()
-{
-	g_list_foreach (window_list, (GFunc) window_update_catalog_list, NULL);
-}
-
-
-void 
-all_windows_update_bookmark_list ()
-{
-	g_list_foreach (window_list, (GFunc) window_update_bookmark_list, NULL);
-}
-
-
-void 
-all_windows_update_browser_options ()
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow  *window = scan->data;
-		
-		window->file_list->enable_thumbs = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE);
-		gth_file_list_set_thumbs_size (window->file_list, eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, 95));
-		gth_file_view_set_view_mode (window->file_list->view,
-					     pref_get_view_mode ());
-		window_update_file_list (window);
-		dir_list_update_underline (window->dir_list);
-	}
-}
-
-
-void 
-all_windows_notify_files_created (GList *list)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_files_created (window, list);
-	}
-}
-
-
-void 
-all_windows_notify_files_deleted (GList *list)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		/* FIXME
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-		*/
-
-		window_notify_files_deleted (window, list);
-	}
-}
-
-
-void 
-all_windows_notify_files_changed (GList *list)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_files_changed (window, list);
-	}
-}
-
-
-void 
-all_windows_notify_cat_files_added (const char *catalog_path,
-				    GList      *list)
-{
-	GList *scan;
-
-	if (list == NULL)
-		return;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_cat_files_added (window, catalog_path, list);
-	}
-}
-
-
-void 
-all_windows_notify_cat_files_deleted (const char *catalog_path,
-				      GList      *list)
-{
-	GList *scan;
-
-	if (list == NULL)
-		return;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_cat_files_deleted (window, catalog_path, list);
-	}
-}
-
-
-void 
-all_windows_notify_file_rename (const gchar *old_name,
-				const gchar *new_name)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_file_rename (window, old_name, new_name);
-	}
-}
-
-
-void
-all_windows_notify_files_rename (GList       *old_names,
-				 GList       *new_names)
-{
-	GList *o_scan, *n_scan;
-
-	for (o_scan = old_names, n_scan = new_names; o_scan && n_scan;) {
-		const char *old_name = o_scan->data;
-		const char *new_name = n_scan->data;
-
-		all_windows_notify_file_rename (old_name, new_name);
-		
-		o_scan = o_scan->next;
-		n_scan = n_scan->next;
-	}
-}
-
-
-void 
-all_windows_notify_directory_rename (const gchar *old_name,
-				     const gchar *new_name)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_rename (window, old_name, new_name);
-	}
-}
-
-
-void 
-all_windows_notify_directory_delete (const gchar *path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_delete (window, path);
-	}
-}
-
-
-void 
-all_windows_notify_directory_new (const gchar *path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_directory_new (window, path);
-	}
-}
-
-
-void
-all_windows_notify_catalog_rename (const gchar *oldname,
-				   const gchar *newname)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_rename (window, oldname, newname);
-	}
-}
-
-
-void
-all_windows_notify_catalog_new (const gchar *path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_new (window, path);
-	}
-}
-
-
-void
-all_windows_notify_catalog_delete (const gchar *path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_catalog_delete (window, path);
-	}
-}
-
-
-void 
-all_windows_notify_update_comment (const gchar *filename)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_update_comment (window, filename);
-	}
-}
-
-
-void 
-all_windows_notify_update_directory (const gchar *dir_path)
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) &&
-		    window->monitor_enabled)
-			continue;
-
-		window_notify_update_directory (window, dir_path);
-	}
-}
-
-
-void
-all_windows_notify_update_icon_theme ()
-{
-	GList *scan;
-
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_notify_update_icon_theme (window);
-	}
-}
-
-
-void
-all_windows_remove_monitor ()
-{
-	GList *scan;
-	
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_remove_monitor (window);
-	}
-}
-
-
-void
-all_windows_add_monitor ()
-{
-	GList *scan;
-	
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		window_add_monitor (window);
-	}
-}
-
-
-GdkPixbuf *
-get_folder_pixbuf (double icon_size)
-{
-	GdkPixbuf       *pixbuf = NULL;
-	static gboolean  scale = TRUE;
-
-	if (folder_pixbuf == NULL) {
-		char *icon_path;
-		
-		icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-							  ICON_NAME_DIRECTORY,
-							  icon_size,
-							  NULL,
-							  NULL);
-
-		if (icon_path == NULL) {
-			folder_pixbuf = gdk_pixbuf_new_from_inline (-1, 
-								    dir_16_rgba, 
-								    FALSE, 
-								    NULL);
-			scale = FALSE;
-
-		} else {
-			folder_pixbuf = gdk_pixbuf_new_from_file (icon_path, 
-								  NULL);
-			g_free (icon_path);
-		}
-	}
-
-	/* Scale keeping aspect ratio. */
-
- 	if (! scale) {
-		g_object_ref (folder_pixbuf);
-		return folder_pixbuf;
-	}
-	
-	if (folder_pixbuf != NULL) {
-		int w, h;
-		
-		w = gdk_pixbuf_get_width (folder_pixbuf);
-		h = gdk_pixbuf_get_height (folder_pixbuf);
-		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
-			pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
-							  w,
-							  h,
-							  GDK_INTERP_BILINEAR);
-		else {
-			pixbuf = folder_pixbuf;
-			g_object_ref (pixbuf);
-		}
-	}
-
-	return pixbuf;
-}
-
-
-
-
 /* SM support */
 
-/* The master client we use for SM */
-static GnomeClient *master_client = NULL;
 
 /* argv[0] from main(); used as the command to restart the program */
 static const char *program_argv0 = NULL;
 
+/* The master client we use for SM */
+static GnomeClient *master_client = NULL;
+
 
 static void
 save_session (GnomeClient *client)
 {
 	const char  *prefix;
-	GList        *scan;
+	GList       *scan;
 	int          i = 0;
 
 	prefix = gnome_client_get_config_prefix (client);
 	gnome_config_push_prefix (prefix);
 
-	for (scan = window_list; scan; scan = scan->next) {
-		GThumbWindow *window = scan->data;
-		char         *location = NULL;
-		char         *key;
+	for (scan = gth_window_get_window_list (); scan; scan = scan->next) {
+		GthWindow  *window = scan->data;
+		char       *uri = NULL;
+		const char *location;
+		char       *key;
 
-		if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
-		    && (window->dir_list != NULL)
-		    && (window->dir_list->path != NULL))
-			location = g_strconcat ("file://",
-						window->dir_list->path,
-						NULL);
-		else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
-			 && (window->catalog_path != NULL))
-			location = g_strconcat ("catalog://",
-						window->catalog_path,
-						NULL);
-		
-		if (location == NULL)
+		if (GTH_IS_VIEWER (window)) {
+			location = gth_window_get_image_filename (window);
+			if (location == NULL)
+				continue;
+			uri = get_uri_from_path (location);
+
+		} else {
+			GthBrowser *browser = (GthBrowser*) window;
+
+			switch (gth_browser_get_sidebar_content (browser)) {
+			case GTH_SIDEBAR_DIR_LIST:
+				location = gth_browser_get_current_directory (browser);
+				if (location == NULL)
+					continue;
+				uri = get_uri_from_path (location);
+				break;
+			
+			case GTH_SIDEBAR_CATALOG_LIST:
+				location = gth_browser_get_current_catalog (browser);
+				if (location == NULL)
+					continue;
+				uri = get_uri_from_path (location);
+				break;
+				
+			default:
+				break;
+			}
+		}
+
+		if (uri == NULL)
 			continue;
 
 		key = g_strdup_printf ("Session/location%d", i);
-		gnome_config_set_string (key, location);
+		gnome_config_set_string (key, uri);
 
+		g_free (uri);
 		g_free (key);
-		g_free (location);
 
 		i++;
 	}
@@ -984,8 +356,207 @@ session_is_restored (void)
 }
 
 
-static gboolean
-load_session (void)
+/* Initialize application data. */
+static void 
+initialize_data (poptContext pctx)
+{
+	const char **argv;
+	int          argc;
+	char        *current_dir;
+	char        *path;
+	int          i;
+
+	convert_to_new_comment_system (); 
+	create_default_categories_if_needed ();
+
+	eel_gconf_monitor_add ("/apps/gthumb");
+
+	monitor = gth_monitor_new ();
+
+	/* Icon theme */
+
+	icon_theme = gnome_icon_theme_new ();
+	gnome_icon_theme_set_allow_svg (icon_theme, TRUE);
+	g_signal_connect (icon_theme,
+			  "changed",
+			  G_CALLBACK (theme_changed_cb),
+			  NULL);
+
+	/* Default windows icon */
+
+	init_icon_pixbufs ();
+	gtk_window_set_default_icon_name ("gthumb");
+	gtk_window_set_default_icon_from_file (PIXMAPSDIR "gthumb.png", NULL);
+
+	/**/
+
+	init_session ("gthumb");
+	if (session_is_restored ()) 
+		return;
+
+	/* Parse command line arguments. */
+
+	argv = poptGetArgs (pctx);
+
+	if (argv == NULL)
+		return;
+
+	for (argc = 0; argv[argc] != NULL; argc++) 
+		;
+
+	if (argc == 0)
+		return;
+
+	file_urls = g_new0 (char*, argc);
+	dir_urls = g_new0 (char*, argc);
+
+	n_file_urls = 0;
+	n_dir_urls = 0;
+
+	current_dir = g_get_current_dir ();
+	for (i = 0; i < argc; i++) {
+		char     *tmp1, *tmp2;
+		gboolean  is_dir;
+
+		if (! g_path_is_absolute (argv[i]))
+			tmp1 = g_strconcat (current_dir, "/", argv[i], NULL);
+		else
+			tmp1 = g_strdup (argv[i]);
+
+		tmp2= remove_special_dirs_from_path (tmp1);
+
+		if (! g_path_is_absolute (tmp2))
+			path = g_strconcat (current_dir, "/", tmp2, NULL);
+		else {
+			path = tmp2;
+			tmp2 = NULL;
+		}
+		
+		g_free (tmp1);
+		g_free (tmp2);
+
+		if (path_is_dir (path))
+			is_dir = TRUE;
+		else if (path_is_file (path))
+			is_dir = FALSE;
+		else {
+			g_free (path);
+			continue;
+		}
+
+		if (is_dir) {
+			dir_urls[n_dir_urls++] = get_uri_from_path (path);
+			g_free (path);
+		} else
+			file_urls[n_file_urls++] = path;
+	}
+
+	if ((n_file_urls == 1) && (file_urls[0] != NULL)) {
+		view_single_image = TRUE;
+
+	} else if (n_file_urls > 1) {
+		/* Create a catalog with the command line list. */
+		Catalog *catalog;
+		char    *catalog_path;
+		char    *catalog_name, *catalog_name_utf8;
+
+		catalog = catalog_new ();
+
+		catalog_name_utf8 = g_strconcat (_("Command Line"),
+						 CATALOG_EXT,
+						 NULL);
+		catalog_name = g_filename_from_utf8 (catalog_name_utf8, -1, 0, 0, 0);
+		catalog_path = get_catalog_full_path (catalog_name);
+		g_free (catalog_name);
+		g_free (catalog_name_utf8);
+
+		catalog_set_path (catalog, catalog_path);
+		g_free (catalog_path);
+
+		for (i = 0; i < n_file_urls; i++) 
+			catalog_add_item (catalog, file_urls[i]);
+
+		catalog_write_to_disk (catalog, NULL);
+		catalog_free (catalog);
+
+		view_comline_catalog = TRUE;
+	}
+
+	g_free (current_dir);
+}
+
+
+/* Free application data. */
+static void 
+release_data (void)
+{
+	if (gth_application != NULL)
+                bonobo_object_unref (gth_application);
+
+	free_icon_pixbufs ();
+
+	g_object_unref (icon_theme);
+
+	if (monitor != NULL) {
+		g_object_unref (monitor);
+		monitor = NULL;
+	}
+
+	preferences_release ();
+	eel_global_client_free ();
+}
+
+
+static void 
+open_viewer_window (const char               *uri,
+		    gboolean                  use_factory,
+		    GNOME_GThumb_Application  app,
+		    CORBA_Environment        *env)
+{
+	GtkWidget *current_window;
+
+	if (use_factory) {
+		if (uri == NULL)
+			uri = "";
+		GNOME_GThumb_Application_open_viewer (app, uri, env);
+		return;
+	}
+
+	current_window = gth_viewer_new (uri);
+	gtk_widget_show (current_window);
+	if (first_window == NULL)
+		first_window = current_window;
+}
+
+
+static void 
+open_browser_window (const char               *uri,
+		     gboolean                  show_window,
+		     gboolean                  use_factory,
+		     GNOME_GThumb_Application  app,
+		     CORBA_Environment        *env)
+{
+	GtkWidget *current_window;
+
+	if (use_factory) {
+		if (uri == NULL)
+			uri = "";
+		GNOME_GThumb_Application_open_browser (app, uri, env);
+		return;
+	}
+
+	current_window = gth_browser_new (uri);
+	if (show_window)
+		gtk_widget_show (current_window);
+	if (first_window == NULL)
+		first_window = current_window;
+}
+
+
+static void
+load_session (gboolean                  use_factory,
+	      GNOME_GThumb_Application  app,
+	      CORBA_Environment        *env)
 {
 	int i, n;
 	
@@ -993,25 +564,412 @@ load_session (void)
 
 	n = gnome_config_get_int ("Session/locations");
 	for (i = 0; i < n; i++) {
-		GThumbWindow *window;
-		char         *key;
-		char         *location;
+		char *key;
+		char *location;
 
 		key = g_strdup_printf ("Session/location%d", i);
 		location = gnome_config_get_string (key);
-		g_free (key);
 
-		preferences_set_startup_location (location);
-
-		window = window_new ();
-		gtk_widget_show (window->app);
+		if (uri_scheme_is_file (location) && path_is_file (location))
+			open_viewer_window (location, use_factory, app, env);
+		else
+			open_browser_window (location, TRUE, use_factory, app, env);
 
 		g_free (location);
+		g_free (key);
 	}
 
 	gnome_config_pop_prefix ();
+}
 
-	return TRUE;
+
+/* Create the windows. */
+static void 
+prepare_app (void)
+{
+	CORBA_Object              factory;
+	gboolean                  use_factory = FALSE;
+	CORBA_Environment         env;
+	GNOME_GThumb_Application  app;
+	int                       i;
+	
+	factory = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application_Factory",
+                                                      Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
+                                                      NULL, NULL);
+
+	if (factory != NULL) {
+		use_factory = TRUE;
+		CORBA_exception_init (&env);
+		app = bonobo_activation_activate_from_id ("OAFIID:GNOME_GThumb_Application", 0, NULL, &env);
+	}
+
+	if (session_is_restored ()) {
+		load_session (use_factory, app, &env);
+		return;
+	}
+
+	if (! view_comline_catalog && (n_dir_urls == 0) && (n_file_urls == 0)) {
+		if (ImportPhotos)
+			preferences_set_startup_location (NULL);
+		open_browser_window (NULL, !ImportPhotos, use_factory, app, &env);
+	}
+
+	if (view_single_image) {
+		if (use_factory && eel_gconf_get_boolean (PREF_SINGLE_WINDOW, FALSE))
+			GNOME_GThumb_Application_load_image (app, file_urls[0], &env);
+		else
+			open_viewer_window (file_urls[0], use_factory, app, &env);
+
+	} else if (view_comline_catalog) {
+		char *catalog_uri;
+		char *catalog_path;
+		char *catalog_name;
+
+		ViewFirstImage = TRUE;
+		HideSidebar = TRUE;
+
+		catalog_name = g_strconcat (_("Command Line"), CATALOG_EXT, NULL);
+		catalog_path = get_catalog_full_path (catalog_name);
+		catalog_uri = g_strconcat ("catalog://", catalog_path, NULL);
+
+		open_browser_window (catalog_uri, TRUE, use_factory, app, &env);
+
+		g_free (catalog_name);
+		g_free (catalog_path);
+		g_free (catalog_uri);
+	}
+
+	for (i = 0; i < n_dir_urls; i++) {
+		/* Go to the specified directory. */
+		open_browser_window (dir_urls[i], TRUE, use_factory, app, &env);
+	}
+
+	/* Free urls. */
+
+	if (n_file_urls > 0) {
+		for (i = 0; i < n_file_urls; i++)
+			g_free (file_urls[i]);
+		g_free (file_urls);
+	}
+
+	if (n_dir_urls > 0) {
+		for (i = 0; i < n_dir_urls; i++)
+			g_free (dir_urls[i]);
+		g_free (dir_urls);
+	}
+
+	/**/
+
+	if (use_factory) {
+		bonobo_object_release_unref (app, &env);
+		CORBA_exception_free (&env);
+		gdk_notify_startup_complete ();
+		exit (0);
+	} else
+		gth_application = gth_application_new (gdk_screen_get_default ());
+}
+
+
+static gboolean 
+check_whether_to_import_photos (gpointer data) 
+{
+	if (ImportPhotos 
+	    && (first_window != NULL) 
+	    && GTH_IS_BROWSER (first_window))
+		gth_browser_activate_action_file_camera_import (NULL, GTH_BROWSER (first_window));
+
+	return FALSE;
+}
+
+
+int 
+main (int   argc, 
+      char *argv[])
+{
+	GnomeProgram *program;
+	GValue        value = { 0 };
+	poptContext   pctx;
+
+	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
+	program = gnome_program_init ("gthumb", VERSION,
+				      LIBGNOMEUI_MODULE, 
+				      argc, argv,
+				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_HUMAN_READABLE_NAME, _("gThumb"),
+				      GNOME_PARAM_APP_PREFIX, GTHUMB_PREFIX,
+				      GNOME_PARAM_APP_SYSCONFDIR, GTHUMB_SYSCONFDIR,
+				      GNOME_PARAM_APP_DATADIR, GTHUMB_DATADIR,
+				      GNOME_PARAM_APP_LIBDIR, GTHUMB_LIBDIR,
+				      NULL);
+
+	if (! g_thread_supported ()) {
+		g_thread_init (NULL);
+		gdk_threads_init ();
+	}
+
+	g_object_get_property (G_OBJECT (program),
+			       GNOME_PARAM_POPT_CONTEXT,
+			       g_value_init (&value, G_TYPE_POINTER));
+
+	pctx = g_value_get_pointer (&value);
+	glade_gnome_init ();
+	gthumb_init ();
+	initialize_data (pctx);
+	poptFreeContext (pctx);
+	prepare_app ();
+	g_idle_add (check_whether_to_import_photos, NULL);
+
+	gtk_main ();
+
+	release_data ();
+	return 0;
+}
+
+
+/**/
+
+
+void 
+all_windows_update_catalog_list (void)
+{
+	gth_monitor_notify_reload_catalogs (monitor);
+}
+
+
+void 
+all_windows_notify_update_bookmarks (void)
+{
+	gth_monitor_notify_update_bookmarks (monitor);
+}
+
+
+void
+all_windows_notify_cat_files_created (const char *catalog_path,
+				      GList      *list)
+{
+	gth_monitor_notify_update_cat_files (monitor, 
+					     catalog_path, 
+					     GTH_MONITOR_EVENT_CREATED, 
+					     list);
+}
+
+
+void
+all_windows_notify_cat_files_deleted (const char *catalog_path,
+				      GList      *list)
+{
+	gth_monitor_notify_update_cat_files (monitor, 
+					     catalog_path, 
+					     GTH_MONITOR_EVENT_DELETED, 
+					     list);
+}
+
+
+void 
+all_windows_notify_files_created (GList *list)
+{
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CREATED, list);
+}
+
+
+void 
+all_windows_notify_files_deleted (GList *list)
+{
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_DELETED, list);
+}
+
+
+void 
+all_windows_notify_files_changed (GList *list)
+{
+	gth_monitor_notify_update_files (monitor, GTH_MONITOR_EVENT_CHANGED, list);
+}
+
+
+void 
+all_windows_notify_file_rename (const gchar *old_name,
+				const gchar *new_name)
+{
+	gth_monitor_notify_file_renamed (monitor, old_name, new_name);
+}
+
+
+void
+all_windows_notify_files_rename (GList *old_names,
+				 GList *new_names)
+{
+	GList *o_scan, *n_scan;
+
+	for (o_scan = old_names, n_scan = new_names; o_scan && n_scan;) {
+		const char *old_name = o_scan->data;
+		const char *new_name = n_scan->data;
+
+		gth_monitor_notify_file_renamed (monitor, old_name, new_name);
+		all_windows_notify_file_rename (old_name, new_name);
+		
+		o_scan = o_scan->next;
+		n_scan = n_scan->next;
+	}
+}
+
+
+void 
+all_windows_notify_directory_rename (const gchar *old_name,
+				     const gchar *new_name)
+{
+	gth_monitor_notify_directory_renamed (monitor, old_name, new_name);
+}
+
+
+void 
+all_windows_notify_directory_delete (const char *path)
+{
+	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_DELETED);
+}
+
+
+void 
+all_windows_notify_directory_new (const char *path)
+{
+	gth_monitor_notify_update_directory (monitor, path, GTH_MONITOR_EVENT_CREATED);
+}
+
+
+void
+all_windows_notify_catalog_rename (const char *old_name,
+				   const char *new_name)
+{
+	gth_monitor_notify_catalog_renamed (monitor, old_name, new_name);
+}
+
+
+void
+all_windows_notify_catalog_new (const gchar *path)
+{
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CREATED);
+}
+
+
+void
+all_windows_notify_catalog_delete (const gchar *path)
+{
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_DELETED);
+}
+
+
+void
+all_windows_notify_catalog_reordered (const gchar *path)
+{
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CHANGED);
+}
+
+
+void 
+all_windows_notify_update_metadata (const gchar *filename)
+{
+	gth_monitor_notify_update_metadata (monitor, filename);
+}
+
+
+void
+all_windows_notify_update_icon_theme (void)
+{
+	gth_monitor_notify_update_icon_theme (monitor);
+}
+
+
+void
+all_windows_remove_monitor (void)
+{
+	gth_monitor_pause (monitor);
+}
+
+
+void
+all_windows_add_monitor (void)
+{
+	gth_monitor_resume (monitor);
+}
+
+
+static GdkPixbuf *
+get_fs_icon (IconName icon_name,
+	     double   icon_size)
+{
+	GdkPixbuf *pixbuf = NULL;
+	gboolean   scale = TRUE;
+
+	/*
+	if (icon_pixbuf[icon_name] != NULL) {
+		g_object_ref (icon_pixbuf[icon_name]);
+		return icon_pixbuf[icon_name];
+	}
+
+	icon_pixbuf[icon_name] = create_pixbuf (icon_theme,
+						icon_mime_name[icon_name],
+						icon_size);
+	
+	return icon_pixbuf[icon_name];
+	*/
+
+	if (icon_pixbuf[icon_name] == NULL) {
+		const GnomeIconData *icon_data;
+		int                  base_size;
+		char                *icon_path;
+		
+		icon_path = gnome_icon_theme_lookup_icon (icon_theme,
+							  icon_mime_name[icon_name],
+							  icon_size,
+							  &icon_data,
+							  &base_size);
+
+		if (icon_path == NULL) {
+			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1, 
+									     dir_16_rgba, 
+									     FALSE, 
+									     NULL);
+			scale = FALSE;
+		} else {
+			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_file (icon_path, 
+									   NULL);
+			g_free (icon_path);
+		}
+	}
+
+	/* Scale keeping aspect ratio. */
+
+ 	if (! scale) {
+		g_object_ref (icon_pixbuf[icon_name]);
+		return icon_pixbuf[icon_name];
+	}
+
+	if (icon_pixbuf[icon_name] != NULL) {
+		int w, h;
+		
+		w = gdk_pixbuf_get_width (icon_pixbuf[icon_name]);
+		h = gdk_pixbuf_get_height (icon_pixbuf[icon_name]);
+		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
+			pixbuf = gdk_pixbuf_scale_simple (icon_pixbuf[icon_name],
+							  w,
+							  h,
+							  GDK_INTERP_BILINEAR);
+		else {
+			pixbuf = icon_pixbuf[icon_name];
+			g_object_ref (pixbuf);
+		}
+	}
+
+	return pixbuf;
+}
+
+
+GdkPixbuf *
+get_folder_pixbuf (double icon_size)
+{
+	return get_fs_icon (ICON_NAME_DIRECTORY, icon_size);
 }
 
 
@@ -1045,9 +1003,9 @@ folder_is_film (const char *folder)
 	CommentData *cdata;
 	gboolean     film = FALSE;
 
-	folder = pref_util_remove_prefix (folder);
+	folder = remove_scheme_from_uri (folder);
 
-	cdata = comments_load_comment (folder);
+	cdata = comments_load_comment (folder, FALSE);
 	if (cdata != NULL) {
 		int i;
 		for (i = 0; i < cdata->keywords_n; i++)
@@ -1060,3 +1018,64 @@ folder_is_film (const char *folder)
 
 	return film;
 }
+
+
+G_CONST_RETURN char *
+get_stock_id_for_uri (const char *uri)
+{
+	const char *stock_id;
+	char       *home_uri;
+
+	home_uri = g_strconcat ("file://", g_get_home_dir (), NULL);
+
+	if (strcmp (uri, g_get_home_dir ()) == 0) 
+		stock_id = GTK_STOCK_HOME;
+	else if (strcmp (uri, home_uri) == 0) 
+		stock_id = GTK_STOCK_HOME;
+	else if (folder_is_film (uri))
+		stock_id = GTHUMB_STOCK_FILM;
+	else if (uri_scheme_is_catalog (uri)) 
+		stock_id = GTHUMB_STOCK_CATALOG;
+	else if (uri_scheme_is_search (uri))	
+		stock_id = GTHUMB_STOCK_SEARCH;
+	else
+		stock_id = GTK_STOCK_OPEN;
+
+	g_free (home_uri);
+
+	return stock_id;
+}
+
+
+GdkPixbuf *
+get_icon_for_uri (GtkWidget  *widget,
+		  const char *uri)
+{
+	const char *stock_id = NULL;
+	char       *home_uri;
+	int         menu_size;
+
+	menu_size = get_folder_pixbuf_size_for_list (widget);
+
+	home_uri = g_strconcat ("file://", g_get_home_dir (), NULL);
+	if (strcmp (uri, g_get_home_dir ()) == 0) 
+		return get_fs_icon (ICON_NAME_HOME, menu_size);
+	else if (strcmp (uri, home_uri) == 0) 
+		return get_fs_icon (ICON_NAME_HOME, menu_size);
+	g_free (home_uri);
+
+	if ((strcmp (uri, "file://") == 0) || (strcmp (uri, "/") == 0)) 
+		return get_fs_icon (ICON_NAME_HARDDISK, menu_size);
+
+	if (folder_is_film (uri))
+		stock_id = GTHUMB_STOCK_FILM;
+	else if (uri_scheme_is_catalog (uri)) 
+		stock_id = GTHUMB_STOCK_CATALOG;
+	else if (uri_scheme_is_search (uri))	
+		stock_id = GTHUMB_STOCK_SEARCH;
+
+	if (stock_id != NULL)
+		return gtk_widget_render_icon (widget, stock_id, GTK_ICON_SIZE_MENU, NULL); 
+	else
+		return get_fs_icon (ICON_NAME_DIRECTORY, menu_size);
+}
